# Generated by Django 2.0.4 on 2018-06-01 14:01

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('genome', '0002_auto_20180330_0837'),
    ]

    operations = [
        migrations.CreateModel(
            name='GeneList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('label', models.CharField(max_length=50)),
                ('description', models.CharField(blank=True, max_length=255)),
                ('version', models.CharField(max_length=25)),
                ('active', models.BooleanField(default=True)),
                ('genes', models.ManyToManyField(blank=True, related_name='gene_lists', to='genome.Gene')),
            ],
            options={
                'verbose_name': 'Gene List',
                'verbose_name_plural': 'Gene Lists',
            },
        ),
        migrations.AlterUniqueTogether(
            name='genelist',
            unique_together={('label', 'version')},
        ),
    ]
